/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.flood_duration;

import java.util.List;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.DefaultFacet;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.Calculation;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;
import org.dive4elements.river.exports.fixings.FixChartGenerator;
import org.dive4elements.river.jfree.RiverAnnotation;
import org.dive4elements.river.jfree.StickyAxisAnnotation;
import org.dive4elements.river.model.Attribute.AttributeKey;

/**
 * Facet to show W and Q annotation lines of an infrastructure height.
 */
public class FloodDurationInfrastructureFacet extends DefaultFacet {

    private static final long serialVersionUID = 1L;

    /** Own log. */
    private static Logger log = Logger.getLogger(FloodDurationInfrastructureFacet.class);

    private final boolean isW;

    private final int m_resultIndex;

    private final AttributeKey m_riverside;

    public FloodDurationInfrastructureFacet(final String name, final boolean isW, final int resultIndex, final AttributeKey attributeKey, final int facetIndex,
            final String description) {
        this.description = description;
        this.name = name;
        this.index = facetIndex;
        this.m_riverside = attributeKey;
        this.isW = isW;
        this.m_resultIndex = resultIndex;
    }

    /**
     * Returns the data this facet requires.
     */
    @Override
    public Object getData(final Artifact artifact, final CallContext context) {

        log.debug("Get data for flood duration curve infrastructure");

        final D4EArtifact flys = (D4EArtifact) artifact;

        final CalculationResult res = (CalculationResult) flys.compute(context, ComputeType.ADVANCE, false);

        final FloodDurationCalculationResults data = (FloodDurationCalculationResults) res.getData();

        final FloodDurationCalculationResult result = data.getResults().get(this.m_resultIndex);

        final double currentKm = FixChartGenerator.getCurrentKm(context);

        final Calculation problems = new Calculation();

        final FacetCalculator calculator = new FacetCalculator(context);
        final List<StickyAxisAnnotation> annotations = calculator.calcInfrastructureAnnotations(problems, currentKm, this.isW, result, this.m_riverside);

        return new RiverAnnotation(this.description, annotations);
    }

    /**
     * Create a deep copy of this Facet.
     *
     * @return a deep copy.
     */
    @Override
    public FloodDurationInfrastructureFacet deepCopy() {
        final FloodDurationInfrastructureFacet copy = new FloodDurationInfrastructureFacet(this.name, this.isW, this.m_resultIndex, this.m_riverside,
                this.index, this.description);
        copy.set(this);
        return copy;
    }
}
